# Generated by roxygen2: do not edit by hand

S3method(IPA,CauseSpecificCox)
S3method(IPA,coxph)
S3method(IPA,default)
S3method(IPA,glm)
S3method(Score,list)
S3method(anova,ate)
S3method(as.data.table,ate)
S3method(as.data.table,influenceTest)
S3method(as.data.table,predictCSC)
S3method(as.data.table,predictCox)
S3method(as.data.table,transformCIBP)
S3method(autoplot,Score)
S3method(autoplot,ate)
S3method(autoplot,predictCSC)
S3method(autoplot,predictCox)
S3method(boxplot,Score)
S3method(coef,CauseSpecificCox)
S3method(coef,ate)
S3method(coef,riskRegression)
S3method(coef,wglm)
S3method(confint,ate)
S3method(confint,influenceTest)
S3method(confint,predictCSC)
S3method(confint,predictCox)
S3method(confint,wglm)
S3method(coxBaseEstimator,GLMnet)
S3method(coxBaseEstimator,coxph)
S3method(coxBaseEstimator,phreg)
S3method(coxBaseEstimator,prodlim)
S3method(coxCenter,coxph)
S3method(coxCenter,cph)
S3method(coxCenter,phreg)
S3method(coxFormula,GLMnet)
S3method(coxFormula,coxph)
S3method(coxFormula,cph)
S3method(coxFormula,glm)
S3method(coxFormula,phreg)
S3method(coxFormula,prodlim)
S3method(coxLP,GLMnet)
S3method(coxLP,coxph)
S3method(coxLP,cph)
S3method(coxLP,phreg)
S3method(coxLP,prodlim)
S3method(coxModelFrame,GLMnet)
S3method(coxModelFrame,coxph)
S3method(coxModelFrame,cph)
S3method(coxModelFrame,phreg)
S3method(coxModelFrame,prodlim)
S3method(coxN,CauseSpecificCox)
S3method(coxN,GLMnet)
S3method(coxN,coxph)
S3method(coxN,cph)
S3method(coxN,default)
S3method(coxN,glm)
S3method(coxN,phreg)
S3method(coxN,prodlim)
S3method(coxSpecial,GLMnet)
S3method(coxSpecial,coxph)
S3method(coxSpecial,cph)
S3method(coxSpecial,phreg)
S3method(coxSpecial,prodlim)
S3method(coxStrata,GLMnet)
S3method(coxStrata,coxph)
S3method(coxStrata,cph)
S3method(coxStrata,phreg)
S3method(coxStrata,prodlim)
S3method(coxStrataLevel,coxph)
S3method(coxStrataLevel,cph)
S3method(coxStrataLevel,phreg)
S3method(coxStrataLevel,prodlim)
S3method(coxVarCov,coxph)
S3method(coxVarCov,cph)
S3method(coxVarCov,phreg)
S3method(coxVarCov,prodlim)
S3method(formula,wglm)
S3method(iid,wglm)
S3method(iidCox,CauseSpecificCox)
S3method(iidCox,coxph)
S3method(iidCox,cph)
S3method(iidCox,phreg)
S3method(iidCox,prodlim)
S3method(influenceTest,default)
S3method(influenceTest,list)
S3method(information,wglm)
S3method(ipcw,cox)
S3method(ipcw,forest)
S3method(ipcw,marginal)
S3method(ipcw,none)
S3method(ipcw,nonpar)
S3method(ipcw,rfsrc)
S3method(is.iidCox,CauseSpecificCox)
S3method(is.iidCox,coxph)
S3method(is.iidCox,cph)
S3method(is.iidCox,default)
S3method(is.iidCox,phreg)
S3method(is.iidCox,prodlim)
S3method(model.tables,ate)
S3method(model.tables,wglm)
S3method(nobs,CauseSpecificCox)
S3method(nobs,multinom)
S3method(nobs,wglm)
S3method(plot,riskRegression)
S3method(predict,CauseSpecificCox)
S3method(predict,FGR)
S3method(predict,riskRegression)
S3method(predict,survfit)
S3method(predictRisk,ARR)
S3method(predictRisk,BinaryTree)
S3method(predictRisk,CauseSpecificCox)
S3method(predictRisk,Cforest)
S3method(predictRisk,CoxConfidential)
S3method(predictRisk,Ctree)
S3method(predictRisk,FGR)
S3method(predictRisk,GLMnet)
S3method(predictRisk,Hal9001)
S3method(predictRisk,SmcFcs)
S3method(predictRisk,SuperPredictor)
S3method(predictRisk,aalen)
S3method(predictRisk,comprisk)
S3method(predictRisk,cox.aalen)
S3method(predictRisk,coxph)
S3method(predictRisk,coxph.penal)
S3method(predictRisk,cph)
S3method(predictRisk,default)
S3method(predictRisk,double)
S3method(predictRisk,factor)
S3method(predictRisk,flexsurvreg)
S3method(predictRisk,formula)
S3method(predictRisk,gbm)
S3method(predictRisk,glm)
S3method(predictRisk,integer)
S3method(predictRisk,lrm)
S3method(predictRisk,matrix)
S3method(predictRisk,multinom)
S3method(predictRisk,numeric)
S3method(predictRisk,penfitS3)
S3method(predictRisk,prodlim)
S3method(predictRisk,psm)
S3method(predictRisk,randomForest)
S3method(predictRisk,ranger)
S3method(predictRisk,rfsrc)
S3method(predictRisk,riskRegression)
S3method(predictRisk,rpart)
S3method(predictRisk,selectCox)
S3method(predictRisk,singleEventCB)
S3method(predictRisk,survfit)
S3method(predictRisk,survreg)
S3method(predictRisk,wglm)
S3method(print,CauseSpecificCox)
S3method(print,FGR)
S3method(print,GLMnet)
S3method(print,IPA)
S3method(print,Score)
S3method(print,ate)
S3method(print,influenceTest)
S3method(print,predictCSC)
S3method(print,predictCox)
S3method(print,riskRegression)
S3method(print,scoreAUC)
S3method(print,scoreBrier)
S3method(print,scoreIPA)
S3method(print,subjectWeights)
S3method(print,synth_code)
S3method(print,wglm)
S3method(rsquared,CauseSpecificCox)
S3method(rsquared,coxph)
S3method(rsquared,default)
S3method(rsquared,glm)
S3method(score,wglm)
S3method(subjectWeights,cox)
S3method(subjectWeights,forest)
S3method(subjectWeights,km)
S3method(subjectWeights,marginal)
S3method(subjectWeights,none)
S3method(subjectWeights,nonpar)
S3method(subsetIndex,default)
S3method(subsetIndex,matrix)
S3method(summary,FGR)
S3method(summary,Score)
S3method(summary,ate)
S3method(summary,riskRegression)
S3method(summary,wglm)
S3method(synthesize,formula)
S3method(synthesize,lvm)
S3method(vcov,ate)
S3method(vcov,wglm)
S3method(weights,wglm)
export(ARR)
export(CSC)
export(Cforest)
export(Ctree)
export(FGR)
export(GLMnet)
export(Hal9001)
export(IPA)
export(LRR)
export(Score)
export(Score.list)
export(SuperPredictor)
export(ate)
export(baseHaz_cpp)
export(boot2pvalue)
export(colCenter_cpp)
export(colCumSum)
export(colMultiply_cpp)
export(colScale_cpp)
export(coxBaseEstimator)
export(coxCenter)
export(coxFormula)
export(coxLP)
export(coxModelFrame)
export(coxN)
export(coxSpecial)
export(coxStrata)
export(coxStrataLevel)
export(coxVarCov)
export(coxVariableName)
export(discreteRoot)
export(getSplitMethod)
export(iidCox)
export(influenceTest)
export(ipcw)
export(is.iidCox)
export(penalizedS3)
export(plotAUC)
export(plotBrier)
export(plotCalibration)
export(plotEffects)
export(plotPredictRisk)
export(plotROC)
export(plotRisk)
export(predictCox)
export(predictCoxPL)
export(predictRisk)
export(riskLevelPlot)
export(riskRegression)
export(riskRegression.options)
export(rowCenter_cpp)
export(rowCumSum)
export(rowMultiply_cpp)
export(rowScale_cpp)
export(rowSumsCrossprod)
export(rsquared)
export(sampleData)
export(saveCoxConfidential)
export(saveSynth)
export(selectCox)
export(simActiveSurveillance)
export(simMelanoma)
export(simPBC)
export(simsynth)
export(subjectWeights)
export(subsetIndex)
export(summary.Score)
export(synthesize)
export(synthesize.formula)
export(synthesize.lvm)
export(transformCIBP)
export(wglm)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(cmprsk,predict.crr)
importFrom(data.table,".N")
importFrom(data.table,".SD")
importFrom(data.table,":=")
importFrom(data.table,as.data.table)
importFrom(data.table,copy)
importFrom(data.table,data.table)
importFrom(data.table,dcast)
importFrom(data.table,is.data.table)
importFrom(data.table,melt)
importFrom(data.table,rbindlist)
importFrom(data.table,set)
importFrom(data.table,setcolorder)
importFrom(data.table,setkey)
importFrom(data.table,setkeyv)
importFrom(data.table,setnames)
importFrom(data.table,setorder)
importFrom(data.table,setorderv)
importFrom(foreach,"%do%")
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(ggplot2,"%+replace%")
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,autoplot)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,geom_errorbar)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_continuous)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_linetype_manual)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,unit)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(glmnet,cv.glmnet)
importFrom(glmnet,glmnet)
importFrom(grDevices,col2rgb)
importFrom(grDevices,gray)
importFrom(grDevices,rainbow)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,boxplot)
importFrom(graphics,bxp)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(lava,"regression<-")
importFrom(lava,"transform<-")
importFrom(lava,endogenous)
importFrom(lava,exogenous)
importFrom(lava,iid)
importFrom(lava,information)
importFrom(lava,score)
importFrom(lava,sim)
importFrom(mets,phreg)
importFrom(prodlim,Hist)
importFrom(prodlim,dimColor)
importFrom(prodlim,jackknife)
importFrom(prodlim,prodlim)
importFrom(prodlim,sindex)
importFrom(rms,cph)
importFrom(rms,strat)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,confint)
importFrom(stats,cov)
importFrom(stats,delete.response)
importFrom(stats,drop.terms)
importFrom(stats,family)
importFrom(stats,formula)
importFrom(stats,get_all_vars)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,model.tables)
importFrom(stats,na.fail)
importFrom(stats,na.omit)
importFrom(stats,nobs)
importFrom(stats,optim)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,reformulate)
importFrom(stats,rexp)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,smooth)
importFrom(stats,terms)
importFrom(stats,terms.formula)
importFrom(stats,time)
importFrom(stats,uniroot)
importFrom(stats,update)
importFrom(stats,update.formula)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(stats,weights)
importFrom(stats,wilcox.test)
importFrom(survival,Surv)
importFrom(survival,coxph)
importFrom(survival,strata)
importFrom(survival,survreg)
importFrom(timereg,Event)
importFrom(timereg,comp.risk)
importFrom(utils,capture.output)
importFrom(utils,find)
importFrom(utils,head)
importFrom(utils,select.list)
importFrom(utils,setTxtProgressBar)
importFrom(utils,tail)
importFrom(utils,txtProgressBar)
useDynLib(riskRegression, .registration=TRUE)
