% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coxN.R
\name{coxN}
\alias{coxN}
\alias{coxN.cph}
\alias{coxN.coxph}
\alias{coxN.default}
\alias{coxN.phreg}
\alias{coxN.CauseSpecificCox}
\alias{coxN.glm}
\alias{coxN.prodlim}
\alias{coxN.GLMnet}
\title{Extract the number of observations from a Cox model}
\usage{
coxN(object)

\method{coxN}{cph}(object)

\method{coxN}{coxph}(object)

\method{coxN}{default}(object)

\method{coxN}{phreg}(object)

\method{coxN}{CauseSpecificCox}(object)

\method{coxN}{glm}(object)

\method{coxN}{prodlim}(object)

\method{coxN}{GLMnet}(object)
}
\arguments{
\item{object}{The fitted Cox regression model object either
obtained with \code{coxph} (survival package), \code{cph}
(rms package), or \code{phreg} (mets package).}
}
\description{
Extract the number of observations from a Cox model
}
\author{
Brice Ozenne broz@sund.ku.dk
}
